module DefaultValidations
  class ValidationsCreator
    class << self

      def create_validations
        models_to_validate.each do |klass|
          create_string_validations_for_class(klass) if DefaultValidations.validate_strings
          create_integer_validations_for_class(klass) if DefaultValidations.validate_integers
        end
      end


      def create_string_validations_for_class(klass)
        not_validated_attributes = not_validated_string_attributes_for_class(klass)
        if not_validated_attributes.present?
          klass.class_eval do
            validates *not_validated_attributes, length: { maximum: DefaultValidations.string_length }
          end
        end
      end

      def create_integer_validations_for_class(klass)
        not_validated_attributes = not_validated_integer_attributes_for_class(klass)
        if not_validated_attributes.present?
          klass.class_eval do
            validates *not_validated_attributes, numericality: { greater_than: -DefaultValidations.integer_max_value, less_than: DefaultValidations.integer_max_value }
          end
        end
      end

      private
      def all_models_with_table
        ActiveRecord::Base.descendants.select(&:table_exists?)
      end

      def models_to_validate
       @@models ||= DefaultValidations.models.empty? ? (all_models_with_table - DefaultValidations.excluded_models)
                                                       : DefaultValidations.models
      end

      def carrier_wave_uploader_attributes(klass)
        DefaultValidations.ignore_carrierwave_uploaders ? klass.uploaders.keys.map(&:to_s) : []
      end

      def not_validated_string_attributes_for_class(klass)
        not_validated_attributes = klass.column_names.select do |name|
          klass.columns_hash[name].type == :string && !klass.validators_on(name).map(&:class).include?(ActiveModel::Validations::LengthValidator)
        end - carrier_wave_uploader_attributes(klass)
      end

      def not_validated_integer_attributes_for_class(klass)
        not_validated_attributes = klass.column_names.select do |name|
          klass.columns_hash[name].type == :integer && !klass.validators_on(name).map(&:class).include?(ActiveModel::Validations::NumericalityValidator)
        end - excluded_primary_and_foreign_keys_for_class(klass)
      end

      def excluded_primary_and_foreign_keys_for_class(klass)
        klass.reflect_on_all_associations(:belongs_to).map(&:foreign_key) << klass.primary_key
      end
    end
  end
end
